#!/usr/bin/env python3
import os, subprocess, shutil, csv
import pandas as pd
import numpy as np
import datetime as dt
from osgeo import gdal

####------User Set Params--------####
dembuf = 0.1 #buffer DEM extent. Can be increased if ISCE indicates extent is too small.
demstr ='demLat_'
demtype = 'AW3D30_E' #'AW3D30_E' or SRTMGL_1_E , only needed when downloading from opentopography
smost, nmost, wmost, emost = 60.0, 63.0, -118.0, -112.5
pol = 'HV'

####------Set Params--------####
cwd = os.getcwd()
prjdir, zerodir = os.path.split(cwd)
coregdir = os.path.join(prjdir, '1_getoffs')
resampdir = os.path.join(prjdir, '2_coreg')
postprocdir = os.path.join(prjdir, '3_postprocess_stacks')
isceprocdir = os.path.join(prjdir, 'SLC') 
mkdl = [coregdir, resampdir, postprocdir, isceprocdir]
for numd, vald in enumerate(mkdl):
    if os.path.exists(vald) == False:
        os.mkdir(vald)

mSlcl = sorted([f for f in os.listdir('.') if f.endswith('.zip')])
mSlcp = [] 
masterSlczip = mSlcl[0]
print('the reference file is %s' %masterSlczip)

if os.path.exists(os.path.join(cwd,'processed.txt')):
    with open('processed.txt', 'r') as f:
        mSlcp = f.read().splitlines()
    bb = set(mSlcp)
else:
    bb=set()

aa = set(mSlcl)
new2proc = list(aa-bb)
mSlcp.extend(new2proc)

#pols = ['HHHH', 'VVVV', 'HVHV', 'HHHV', 'HHVV', 'HVVV']

#####-----------get dem function

#dembuf: a buffer around the DEM, in units of degrees lat/lon
#smost, nmost, wmost, emost the extents of the roi
def get_dem(dembuf, smost, nmost, wmost, emost):
    demf = [f for f in os.listdir('.') if f.startswith('dem.wgs84')]
    if len(demf)==0 and nmost < 60:
        print('Downloading SRTM1 DEM for SNWE box %s %s %s %s' %(smost, nmost, wmost, emost))
        subprocess.call(['dem.py','-a', 'stitch', '-b', str(smost), str(nmost), str(wmost), str(emost), '-r', '-s', '1', '-c'])

    elif len(demf)==0 and nmost >= 60:
        print('Downloading high latitude DEM for SNWE box %s %s %s %s' %(smost-dembuf, nmost+dembuf, wmost-dembuf, emost+dembuf))
        demlink = 'https://portal.opentopography.org/API/globaldem?demtype='+demtype+'&south='+str(smost-dembuf)+'&north='+str(nmost+dembuf)+'&west='+str(wmost-dembuf)+'&east='+str(emost+dembuf)+'&outputFormat=GTiff'
        print(demlink)

        #cmd = 'curl -X GET "'+demlink+'" -H "accept: */*" --output dem.tif'
        subprocess.call(['curl', '-X', 'GET', demlink, '-H', 'accept: */*', '--output', 'dem.tif'])

        #print('Filling nodata values')
        #subprocess.call(['gdal_fillnodata.py', 'dem.tif'])
        print('Translating to flat ENVI format')
        subprocess.call(['gdal_translate', '-of', 'ENVI', '-ot', 'Int16', 'dem.tif', 'dem.wgs84'])
        print('Making it an ISCE dem')
        subprocess.call(['gdal2isce_xml.py', '-i', 'dem.wgs84'])
        os.remove('dem.tif')

        #for high latitude sites, downloading from opentopographyi
        #Note: if there is an issue with with the ISCE outputs, check DEM first. Located in merged/geom_master/hgt.rdr
    else:
        print('DEM found, skipping download')

print('-----------STEP 1 Downloading DEM--------------')

#get_dem(dembuf, smost, nmost, wmost, emost)        

#update the xml for absolute pathname
demn = [f for f in os.listdir('.') if f.endswith('dem.wgs84')]
print('Fixing Image XML')
subprocess.call(['fixImageXml.py', '-f', '-i', demn[0]])

#####----------iterate over new zip files in cwd-------------#####
cwd = os.getcwd()

if len(new2proc)==0:
    print('No new files to process')
else:
    print('New files to process: %s' %(new2proc))

print('-----------STEP 2 unzip and place data--------------')

os.chdir(cwd)
subprocess.call(['prepSlcALOS2.py', '-i', cwd, '-o', os.path.join(prjdir, isceprocdir), '-p', pol])

for num, val in enumerate(new2proc):
    os.chdir(cwd)
    date = '20' + val.split('-')[1].split('.')[0]
    print(date)
    subprocess.call(['unpackFrame_ALOS2.py', '-i', os.path.join(cwd,date), '-o', os.path.join(prjdir, isceprocdir, date), '-p', pol])


mSlcl = sorted([f for f in os.listdir(os.path.join(prjdir, zerodir, 'ARCHIVED_FILES')) if f.endswith('.zip')])
masterSlc = mSlcl[0]#[:-14]+'HHHH_CX_'+mct+'.slc'
masterdate = '20' + masterSlc.split('-')[1].split('.')[0]

shutil.copytree(os.path.join(isceprocdir, masterdate), os.path.join(postprocdir, masterdate))

###from jupyter...put these into the topo step. do the stack step first!
##geomdir = os.path.join(outdir2, 'geom_master')
##!topo.py -d $demdir -m $outdir3 -o $geomdir

#write log of processed files
if len(new2proc)>0:
    with open('processed.txt', 'w') as f:
        for item in mSlcp:
            f.write("%s\n" %item)

## #remove dem and other files that aren't used
## rmfiles = ('.dem.vrt', '.dem.xml', '.dem', 'x.csv', '_L090_CX_'+crosstalk+'.ann', 'sce.log', '.slc.vrt', '.slc.xml')
## demn = [f for f in os.listdir('.') if f.endswith(rmfiles)]
## for numd, vald in enumerate(demn):
##     os.remove(vald)

