#!/usr/bin/env python3
import os, subprocess, shutil, csv, zipfile, utm
import pandas as pd
import numpy as np
import datetime as dt
from osgeo import gdal

####------User Set Params--------####
geocode = 1     #geocode real valued images
xyres_out = '100,100'   #resolution in meters
intmethod = 'bilinear'#interpolation method; e.g. 'near' or bilinear
epsgproj_out = 'EPSG:32611'

####------Set Params--------####
cwd = os.getcwd()
postprocdir = cwd
prjdir, step1dir = os.path.split(cwd)
zerodir = os.path.join(prjdir, '0_make_ann_dem')
geomdir = os.path.join(prjdir, 'merged', 'geom_master')
mSlcl = sorted([f for f in os.listdir(os.path.join(zerodir, 'ARCHIVED_FILES')) if f.endswith('.zip')])
masterSlc = mSlcl[0]#+'.slc'
masterdate = '20' + masterSlc.split('-')[1].split('.')[0]
#-------------------------------

dirnm = [f for f in os.listdir(postprocdir) if os.path.isdir(f) and f.isnumeric()]

######----------Calculate Magnitudes--------------#####
for num, val in enumerate(dirnm):
    nd = os.path.join(postprocdir, val)
    os.chdir(nd)
    mlcnm = [f for f in os.listdir(nd) if f.endswith('.slc') and not f.endswith('_dB.slc') and not f.endswith('_lin.slc') and not f.endswith('_rtc.slc')]
    print(mlcnm)
    
    for numd, vald in enumerate(mlcnm):
        print('-----------Calculating Corrected Image, Linear Units '+vald+'--------------')
        eq = '--e=10**((10*log10(a*conj(a))-83.0-32.0)/10)'
        ps1 = '--a='+vald
        subprocess.call(['imageMath.py', eq, ps1, '-o', vald[:-4]+'_lin.slc', '-t','FLOAT', '-s', 'BIL']) #BIL BIP
        
os.chdir(postprocdir)
######----------Calculate RTC--------------#####
for num, val in enumerate(dirnm):
    nd = os.path.join(postprocdir, val)
    os.chdir(nd)
    mlcnm = [f for f in os.listdir(nd) if f.endswith('_lin.slc')]
    
    for numd, vald in enumerate(mlcnm):
        print('-----------Apply RTC '+vald+'--------------')
        eq = '--e=(a/cos(f_0*PI/180.))*(tan(b_1*PI/180.)/tan(f_0*PI/180.))*(c==0)'
        ps1 = '--a='+vald
        ps2 = '--b='+geomdir+'/incLocal.rdr'
        ps3 = '--c='+geomdir+'/shadowMask.rdr'
        ps4 = '--f='+geomdir+'/los.rdr'
        subprocess.call(['imageMath.py', eq, ps1, ps2, ps3, ps4, '-o', vald[:-8]+'_rtc.slc', '-t', 'FLOAT', '-s', 'BIL'])

os.chdir(postprocdir)

#######----------Geocode--------------#####
latin = os.path.join(geomdir,'lat.rdr.vrt')
lonin = os.path.join(geomdir,'lon.rdr.vrt')

for num, val in enumerate(dirnm):
    nd = os.path.join(postprocdir, val)
    os.chdir(nd)
    mlcnm = [f for f in os.listdir(nd) if f.endswith('_rtc.slc.vrt')]
    
    for numd, vald in enumerate(mlcnm):
        print('-----------Geocoding '+vald+'--------------')
        outn  = os.path.join(vald[:-8]+'_geo.tif')
        #subprocess.call(['geocode_sk_v2.py', '-i', vald, '-l', latin, '-L', lonin,'-o', vald[:-4]+'_geo.tif', '-c', epsgproj_out, '-s', xyres_out, '-m', intmethod])
        subprocess.call(['geocode_sk_v2.py', '-i', vald, '-l', latin, '-L', lonin,'-o', 'tmp.tif', '-m', intmethod])
        subprocess.call(['gdalwarp', '-t_srs', epsgproj_out, '-tr', '100', '100', '-r', intmethod, 'tmp.tif', vald[:-4]+'_geo.tif'])
        os.remove('tmp.tif') #the gelocation image can be quite large, so I'm deleting it
#!./geocode_sk_v2.py -i $datin -l $latin -L $lonin -o $outn -c $epsgproj_out -s $xyres_out -m $intmethod
