#This script will look through the file_repository_new_runs folder for UAVSAR MLC zip files; create a new project structure for each;
#and copy the python scripts and move the .mlc.zip files to the appropriate subfolders. Then it will run each script in sequence.
#--update: Just download the DEM once, for example with my .ipynb tool. Then put it in docker_new_runs. This script will copy the files over to 0_ann and skip the DEM download
### ----!!! important...need to update the field "file name" in the dem.wgs84.xml each time. Or run fiximagexml

import os, shutil, subprocess, time
import numpy as np
#---user specific input---#
simrun = 0 #set to 1 if using simulated NISAR data as input
#---end user specify input---#

#step0
zerodir = '0_make_ann_dem'
zerofn = '0_alos_dem.py'
zerofn_dem0 = 'dem.hdr'
zerofn_dem1 = 'dem.wgs84'
zerofn_dem2 = 'dem.wgs84.vrt'
zerofn_dem3 = 'dem.wgs84.xml'
#zerofn = '0_make_ann_dem_move_v2.py'

#step1
coregdir = '1_getoffs'
coregfn = '1_isce_stack.py'

#step2
resampdir = '2_coreg'
resampfn = '2_resamp_pol.py'
#resampfn = '2_resamp_pol_v1.py'

#step 3
postprocdir = '3_postprocess_stacks'
postprocfn = '3_uavsar_pp_mlc_v2.py'

cwd = os.getcwd()
os.chdir(cwd)
shutil.copy(os.path.join(cwd, '.netrc'), '/work/.netrc')

maindir, thisdir = os.path.split(cwd)

mlczip = [f for f in os.listdir() if f.endswith('.zip')]

#group the mlc.zip by project...those that start with the same two parameters
prjlst = []
for numd, vald in enumerate(mlczip):
    prj = vald.split('_')[0]#+'_'+vald.split('_')[1]
    prjlst.append(prj)

projects = list(set(prjlst))
print('There are %s projects (%s)' % (len(projects), projects))
print(prj)

for num, val in enumerate(projects):
    prjfiles = [f for f in os.listdir() if f.startswith(val)]
    print('Project %s consists of %s flightlines' % (projects[num], len(prjfiles)))
    if simrun == 1:
        prjdir = os.path.join(maindir, projects[num]+'_sim')
    else:
        prjdir = os.path.join(maindir, projects[num])

    if not os.path.exists(prjdir):
        os.mkdir(prjdir)
        os.mkdir(os.path.join(prjdir,zerodir))
        os.mkdir(os.path.join(prjdir,coregdir))
        os.mkdir(os.path.join(prjdir,resampdir))
        os.mkdir(os.path.join(prjdir,postprocdir))
    
    shutil.copy(os.path.join(cwd, zerofn), os.path.join(prjdir, zerodir, zerofn))
    shutil.copy(os.path.join(cwd, zerofn_dem0), os.path.join(prjdir, zerodir, zerofn_dem0))
    shutil.copy(os.path.join(cwd, zerofn_dem1), os.path.join(prjdir, zerodir, zerofn_dem1))
    shutil.copy(os.path.join(cwd, zerofn_dem2), os.path.join(prjdir, zerodir, zerofn_dem2))
    shutil.copy(os.path.join(cwd, zerofn_dem3), os.path.join(prjdir, zerodir, zerofn_dem3))
    shutil.copy(os.path.join(cwd, coregfn), os.path.join(prjdir, coregdir, coregfn))
    shutil.copy(os.path.join(cwd, resampfn), os.path.join(prjdir, resampdir, resampfn))
    shutil.copy(os.path.join(cwd, postprocfn), os.path.join(prjdir, postprocdir, postprocfn))
   
    for numd, vald in enumerate(prjfiles):
        shutil.move(os.path.join(cwd, vald), os.path.join(prjdir, zerodir, vald))
    
    t0 = time.time()
    print('Running %s for project %s' % (zerofn, val))
    os.chdir(os.path.join(prjdir, zerodir))
    aa = os.getcwd()
    subprocess.call([os.path.join(aa,zerofn)])
    t1 = time.time()
    print('Running %s for project %s' % (coregfn, val))
    os.chdir(os.path.join(prjdir, coregdir))
    aa = os.getcwd()
    subprocess.call([os.path.join(aa, coregfn)])
    t2 = time.time()
    print('Running %s for project %s' % (resampfn, val))
    os.chdir(os.path.join(prjdir, resampdir))
    aa = os.getcwd()
    subprocess.call([os.path.join(aa,resampfn)])
    t3 = time.time()
    print('Running %s for project %s' % (postprocfn, val))
    os.chdir(os.path.join(prjdir, postprocdir))
    aa = os.getcwd()
    subprocess.call([os.path.join(aa, postprocfn)])
    t4 = time.time()
   
    print('0_make_ann_dem took %s seconds' %(np.round(t1-t0,3)))
    print('1_getoffs took %s seconds' %(np.round(t2-t1,3)))
    print('2_coreg took %s seconds' %(np.round(t3-t2,3)))
    print('3_postprocess took %s seconds' %(np.round(t4-t3,3)))
    print('Processing took %s seconds' %(np.round(t4-t0,3)))
    os.chdir(cwd)
